#include <sstream>
#include <qblib/stream.hh>

PrintStreamBase& PrintStreamBase::operator<< (const BAS_INT v)
{
    std::stringstream tmp; tmp << v; WriteWithFitCheck(tmp.str());
}
PrintStreamBase& PrintStreamBase::operator<< (const BAS_LNG v)
{
    std::stringstream tmp; tmp << v; WriteWithFitCheck(tmp.str());
}
PrintStreamBase& PrintStreamBase::operator<< (const BAS_SGN v)
{
    std::stringstream tmp; tmp << v; WriteWithFitCheck(tmp.str());
}
PrintStreamBase& PrintStreamBase::operator<< (const BAS_DBL v)
{
    std::stringstream tmp; tmp << v; WriteWithFitCheck(tmp.str());
}
PrintStreamBase& PrintStreamBase::operator<< (const BAS_STR& v)
{
    WriteWithFitCheck(v);
}
PrintStreamBase& PrintStreamBase::operator<< (const char* v)
{
    WriteWithFitCheck(v);
}
PrintStreamBase& PrintStreamBase::operator<< (const char v)
{
    WriteWithFitCheck(&v, 1);
}
void PrintStreamBase::WriteWithFitCheck(const void* buf, unsigned n)
{
    CheckFit(n);
    Write(buf, n);
}
void PrintStreamBase::DoSPC(int count)
{
    if(count < 0) count = 0;
    int xpos = GetXpos();
    unsigned xsize = GetXsize();
    if(xsize) count %= xsize;
    std::string spaces(count, ' ');
    Write(&spaces[0], spaces.size());
}
void PrintStreamBase::DoTAB(int newpos)
{
    --newpos;
    if(newpos < 0) newpos = 0;
    int xpos = GetXpos();
    unsigned xsize = GetXsize();
    if(xsize) newpos %= xsize;
    
    if(newpos < xpos) { xpos=newpos=0; DoCRLF(); }
    if(newpos > xpos) Write(std::string(newpos-xpos, ' '));
}
void PrintStreamBase::DoNextTAB()
{
    const unsigned ColumnWidth = 14;
    static const char SpaceBuf[ColumnWidth+1] = "              ";
    int xpos = GetXpos();
    int columnpos   = xpos % ColumnWidth;
    int need_spaces = ColumnWidth - columnpos;
    if(CheckFit(need_spaces))
        Write(SpaceBuf, need_spaces);
}
bool PrintStreamBase::CheckFit(int n_columns)
{
    int xsize = GetXsize(); if(!xsize) return true;
    int xpos = GetXpos();
    if(xpos+n_columns < xsize) return true;
    if(xpos > 0) DoCRLF();
    return false;
}
