#include <cstdio>
#include <cstring>

#include <qblib/stream.hh>
#include <qblib/error.hh>
#include <qblib/input.hh>

#include <map>

/* File writing */
class QbFPstream: public PrintStreamBase
{
    FILE* fp;
public:
    QbFPstream(int fd) : fp(fdopen(fd, "wb")) { }
    
    virtual unsigned GetXpos() const
    {
    }
    virtual unsigned GetXsize() const
    {
    }
    virtual void DoCRLF()
    {
    }
    virtual void Write(const void* buf, unsigned n)
    {
    }
};

/* File input */
class QbFIstream: public InputImplement
{
    FILE* fp;
public:
    QbFIstream(int fd) : fp(fdopen(fd, "rb")) { }

    virtual void ReadLine(BAS_STR& target)
    {
    }

    virtual void BeginInput()
    {
        InputError = false;
        // File:
        //   Do nothing. InputTo reads file.
    }

    virtual bool EndInputOkp()
    {
        // File:
        //   Return false if InputTo failed, true otherwise.

        if(InputError) return false;
    }

    virtual BAS_STR Input_N(unsigned n)
    {
        BAS_STR buf(n, '\0');
        size_t res = fread(&buf[0], 1, n, fp);
        
    }

    virtual int InputChar()
    {
    }

    virtual void GotOverflow()
    {
        B_ERROR(6); // overflow
        InputError = true;
    }
};

/* Console I/O */
class QbConsoleStream: public PrintStreamBase, public InputImplement
{
    BAS_STR InputLineBuffer;
    unsigned InputLineBufferPos;
public:
    QbConsoleStream() : PrintStreamBase(), InputImplement() { }
    
    virtual unsigned GetXpos() const
    {
    }
    virtual unsigned GetXsize() const
    {
    }
    virtual void DoCRLF()
    {
    }
    virtual void Write(const void* buf, unsigned n)
    {
    }

    virtual void ReadLine(BAS_STR& target)
    {
    }

    virtual void BeginInput()
    {
        InputError = false;
        // Console:
        //   Read line into buffer. InputTo queries buffer.
        //   FIXME: should keep cursor on same line.
        ReadLine(InputLineBuffer);
        InputLineBufferPos=0;
    }

    virtual bool EndInputOkp()
    {
        // Console:
        //   Return false if InputTo failed.
        //   Return false if buffer is not empty.
        //   Return true otherwise.

        if(InputError) return false;
        IgnoreBlankInput();
        if(InputLineBufferPos < InputLineBuffer.size()) return false;
        return true;
    }

    virtual BAS_STR Input_N(unsigned n)
    {
        // No echoing
    }

    virtual int InputChar()
    {
        if(InputLineBufferPos < InputLineBuffer.size())
        {
            return InputLineBuffer[InputLineBufferPos++];
        }
        return -1;
    }

    virtual void GotOverflow()
    {
        InputError = true;
        *this << "Overflow\n";
    }
};

struct FileDescriptor
{
    QbFPstream fpout;
    QbFIstream fpin;
};
static std::map<int/*qbasic filenum*/, FileDescriptor> fdmap;

PrintStreamBase& GET_STREAM_OUTPUT(int filenumber)
{
    std::map<int, FileDescriptor>::iterator
        i = fdmap.find(filenumber);

    if(i == fdmap.end())
        B_ERROR(52); // bad file number
    
    return i->second.fpout;
}

InputStreamBase& GET_STREAM_INPUT(int filenumber)
{
    std::map<int, FileDescriptor>::iterator
        i = fdmap.find(filenumber);

    if(i == fdmap.end())
        B_ERROR(52); // bad file number
    
    return i->second.fpin;
}

static QbConsoleStream cons;

PrintStreamBase& GET_STDOUT()
{
    static bool tty_out = isatty(1);
    if(!tty_out) { static QbFPstream out(1); return out; }
    return cons;
}
InputStreamBase& GET_STDIN()
{
    static bool tty_in = isatty(0);
    if(!tty_in) { static QbFIstream in(0); return in; }
    return cons;
}


const BAS_STR B_FuncINKEY_S()
{
}
